<?php
/*
Plugin Name: Recent Post Views
Plugin URI: http://cbuckconsulting.com/recent-post-views/
Description: Displays the most recent posts in a widget, how many times they have been viewed over last (n) days, and images for popularity.
Version: 1.1
Author: ChrisBuck
Author URI: http://cbuckconsulting.com
License: GPL2
*/

/*  Copyright 2012  Chris Buck  (email : Chris@CBuckConsulting.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* NOTICE: Up next in a future version, more styling options will be added for the widget. Also shortcode functionality. 

Please rate 5 stars if you love this plugin, at http://wordpress.org/extend/plugins/recent-post-views.

*/

//Widget code-->

class recentPostViews_Widget extends WP_Widget
{
  function recentPostViews_Widget()
  { //Constructor
    $widget_ops = array('classname' => 'recentPostViews_Widget', 'description' => 'Allows you to display recent posts, with the number of views, in the sidebar.');
    $this->WP_Widget('recentPostViews_Widget', 'Recent Post Views', $widget_ops);
  }
 
  function form($instance)
  { //Widget form in backend
    $instance = wp_parse_args((array) $instance, array( 'title' => '', 'timeframe' => '2', 'numposts' => '7', 'category' => 'ALL', 'hotlevel' => '.75', 'hotcolor' => 'd5000'));
    $title = esc_attr($instance['title']);
	$timeframe = intval($instance['timeframe']);
	$numposts = intval($instance['numposts']);
	$category = $instance['category'];
	$hotlevel = $instance['hotlevel'];
	$hotcolor = $instance['hotcolor'];

?><!--Title form element-->
  <p><label for="<?php echo $this->get_field_id('title'); ?>">Title: <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo attribute_escape($title); ?>" /></label></p>
  
  <!--Number of days element-->
  <p><label for="<?php echo $this->get_field_id('timeframe'); ?>">Number of days for "recent" posts: <input class="widefat" id="<?php echo $this->get_field_id('timeframe'); ?>" name="<?php echo $this->get_field_name('timeframe'); ?>" type="text" value="<?php echo attribute_escape($timeframe); ?>" /></label></p>
  <p style="font-size:10px;"><em>How many days old is too old? Expressed as an integer.</em></p>
 
<!--Number of posts element-->
  <p>
			<label for="<?php echo $this->get_field_id('numposts'); ?>"><?php _e('Number of posts to show:', 'recentPostViews'); ?>
				<select name="<?php echo $this->get_field_name('numposts'); ?>" id="<?php echo $this->get_field_id('numposts'); ?>" class="widefat">
					<option value="1"<?php selected(1, $numposts); ?>><?php _e(1, 'recentPostViews'); ?></option>
					<option value="2"<?php selected(2, $numposts); ?>><?php _e(2, 'recentPostViews'); ?></option>
					<option value="3"<?php selected(3, $numposts); ?>><?php _e(3, 'recentPostViews'); ?></option>
					<option value="4"<?php selected(4, $numposts); ?>><?php _e(4, 'recentPostViews'); ?></option>
					<option value="5"<?php selected(5, $numposts); ?>><?php _e(5, 'recentPostViews'); ?></option>
					<option value="6"<?php selected(6, $numposts); ?>><?php _e(6, 'recentPostViews'); ?></option>
					<option value="7"<?php selected(7, $numposts); ?>><?php _e(7, 'recentPostViews'); ?></option>
					<option value="8"<?php selected(8, $numposts); ?>><?php _e(8, 'recentPostViews'); ?></option>
					<option value="9"<?php selected(9, $numposts); ?>><?php _e(9, 'recentPostViews'); ?></option>
					<option value="10"<?php selected(10, $numposts); ?>><?php _e(10, 'recentPostViews'); ?></option>
					<option value="15"<?php selected(15, $numposts); ?>><?php _e(15, 'recentPostViews'); ?></option>
					<option value="20"<?php selected(20, $numposts); ?>><?php _e(20, 'recentPostViews'); ?></option>
					<option value="25"<?php selected(25, $numposts); ?>><?php _e(25, 'recentPostViews'); ?></option>
				</select>
			</label>
		</p>
  
  <!--Category element-->
  <?php //First thing is to get all the category names and ID's for the select list.
  $catargs=array(
  'orderby' => 'name',
  'order' => 'ASC'
  );
  $categories=get_categories($catargs); ?>
  <p>
			<label for="<?php echo $this->get_field_id('category'); ?>"><?php _e('Select a specific category:', 'recentPostViews'); ?>
				<select name="<?php echo $this->get_field_name('category'); ?>" id="<?php echo $this->get_field_id('category'); ?>" class="widefat">
					<option value="ALL"<?php selected('ALL', $category); ?>><?php _e('ALL', 'recentPostViews'); ?></option>
					<?php
					foreach ($categories as $cat) {
						echo '<option value="'.$cat->cat_ID.'"';
						selected ($cat->cat_ID, $category);
						echo '>';
						_e($cat->name, 'recentPostViews');
						echo '</option>';
					}?>
				</select>
			</label>
		</p>
<!--"Hotlevel" element-->
	<p><label for="<?php echo $this->get_field_id('hotlevel'); ?>">Percent threshold for "hot" status:<input class="widefat" id="<?php echo $this->get_field_id('hotlevel'); ?>" name="<?php echo $this->get_field_name('hotlevel'); ?>" type="text" value="<?php echo attribute_escape($hotlevel); ?>" /></label></p>
	<p style="font-size:10px;"><em>Must be expressed as a decimal, e.g., .75.</br>
	This is the percent threshold, of the most views for a post, to be considered "hot." If 0 is the level, all posts are hot. At 1.0, only the most viewed is hot.</em></p>

<!--Hotcolor element-->
  <p><label for="<?php echo $this->get_field_id('hotcolor'); ?>">(Hex) color for "views"?: <input class="widefat" id="<?php echo $this->get_field_id('hotcolor'); ?>" name="<?php echo $this->get_field_name('hotcolor'); ?>" type="text" value="<?php echo attribute_escape($hotcolor); ?>" /></label></p>
  <p style="font-size:10px;"><em>Expressed as a hex color code, no '#', e.g., d5000.</em></p>

<?php
//End widget form
  }
 
  function update($new_instance, $old_instance)
  { //Saves the widget
    $instance = $old_instance;
    $instance['title'] = $new_instance['title'];
	$instance['timeframe'] = $new_instance['timeframe'];
	$instance['numposts'] = $new_instance['numposts'];
	$instance['category'] = $new_instance['category'];
	$instance['hotlevel'] = $new_instance['hotlevel'];
	$instance['hotcolor'] = $new_instance['hotcolor'];
    return $instance;
  }
 
  function widget($args, $instance)
  { //Prints the widget
    extract($args, EXTR_SKIP);
 
    echo $before_widget;
    $title = empty($instance['title']) ? '' : apply_filters('widget_title', $instance['title']);
	$timeframe = empty($instance['timeframe']) ? '' : apply_filters('widget_timeframe', $instance['timeframe']);
	$numposts = empty($instance['numposts']) ? '' : apply_filters('widget_numposts', $instance['numposts']);
	$category = empty($instance['category']) ? '' : apply_filters('widget_catgeory', $instance['category']);
	$hotlevel = empty($instance['hotlevel']) ? '' : apply_filters('widget_hotlevel', $instance['hotlevel']);
	$hotcolor = empty($instance['hotcolor']) ? '' : apply_filters('widget_hotcolor', $instance['hotcolor']);
 
    if (!empty($title))
      echo $before_title . $title . $after_title;;
 
    // Widget Stuff Goes Here...

//Preliminary housekeeping
	global $wpdb;
	$prefix = $wpdb->prefix;
	$homeurl = home_url();
	$imageurl = plugins_url('recent-post-views/');
	$ndays = $timeframe;
	$fontcolor = '#'.$hotcolor;
	
$maxViews = $wpdb->get_row("SELECT SUM(".$prefix."postmeta.meta_value) as 'views'
FROM ".$prefix."postmeta
LEFT JOIN ".$prefix."posts ON ".$prefix."postmeta.post_id = ".$prefix."posts.ID
WHERE ".$prefix."postmeta.meta_key = 'views' AND ".$prefix."posts.post_type = 'post' AND ".$prefix."posts.post_date > DATE_SUB(CURDATE(), INTERVAL ".$ndays." DAY)
GROUP BY ".$prefix."posts.post_title
ORDER BY views DESC
LIMIT 1",ARRAY_A);

//First check if the user selected a category for the widget or not. Then switch out a query based on that info.
if ($category == 'ALL') {
$postviews = $wpdb->get_results("SELECT ".$prefix."posts.post_date, ".$prefix."posts.ID, ".$prefix."posts.post_title, ".$prefix."posts.post_name, SUM(".$prefix."postmeta.meta_value) as 'views'
	FROM ".$prefix."postmeta
	LEFT JOIN ".$prefix."posts ON ".$prefix."postmeta.post_id = ".$prefix."posts.ID
	WHERE ".$prefix."postmeta.meta_key = 'views' AND ".$prefix."posts.post_type = 'post' AND ".$prefix."posts.post_date > DATE_SUB(CURDATE(), INTERVAL ".$ndays." DAY)
	GROUP BY ".$prefix."posts.post_title
	ORDER BY views DESC
	LIMIT ".$numposts."",OBJECT);
}
else {
$postviews = $wpdb->get_results("SELECT ".$prefix."posts.post_date, ".$prefix."posts.ID, ".$prefix."posts.post_title, ".$prefix."posts.post_name, SUM(".$prefix."postmeta.meta_value) as 'views', ".$prefix."term_relationships.term_taxonomy_id, ".$prefix."terms.name
	FROM ".$prefix."postmeta
	LEFT JOIN ".$prefix."posts ON ".$prefix."postmeta.post_id = ".$prefix."posts.ID
        LEFT JOIN ".$prefix."term_relationships ON ".$prefix."posts.ID = ".$prefix."term_relationships.Object_ID
        LEFT JOIN ".$prefix."terms ON ".$prefix."terms.term_id = ".$prefix."term_relationships.term_taxonomy_id
	WHERE ".$prefix."postmeta.meta_key = 'views' AND ".$prefix."posts.post_type = 'post' AND ".$prefix."posts.post_date > DATE_SUB(CURDATE(), INTERVAL ".$ndays." DAY) AND ".$prefix."terms.term_id = ".$category."
	GROUP BY ".$prefix."posts.post_title
	ORDER BY views DESC
	LIMIT ".$numposts."",OBJECT);
}

foreach ($postviews as $widgetvalue) {
//Open the table and the list.
echo'<ul><table>';
echo'<li><tr><td>';

//if the post has the most views, include the "hottest" image.
if ($widgetvalue->views == $maxViews['views'])
	echo '<td><img src="'.$imageurl.'Hottest.png" width="25" height="9" align="top"></td><td><strong><a href="'.$homeurl.'/?p='.$widgetvalue->ID.'">'.$widgetvalue->post_title.'</a></strong> - <font color="'.$fontcolor.'">'.$widgetvalue->views.' views</font></td></tr>';

//if the post has more than the x% of the most views, include the "hot" image.
else if ($widgetvalue->views >= $hotlevel * $maxViews['views'])
	echo '<td><img src="'.$imageurl.'Hot.png" width="25" height="9" align="top"></td><td><strong><a href="'.$homeurl.'/?p='.$widgetvalue->ID.'">'.$widgetvalue->post_title.'</a></strong> - <font color="'.$fontcolor.'">'.$widgetvalue->views.' views</font></td></tr>';

//otherwise, just list the post with a link.
else
	echo '<td></td><td><strong><a href="'.$homeurl.'/?p='.$widgetvalue->ID.'">'.$widgetvalue->post_title.'</a></strong> - '.$widgetvalue->views.' views</td></tr>';

//Close the table and the list.
echo '</li>';
echo '</table></ul>';
}
	
    echo $after_widget;
  }
}
add_action( 'widgets_init', create_function('', 'return register_widget("recentPostViews_Widget");') );


?>